package com.kxs.common.security.service;


import com.kxs.common.core.constant.SecurityConstants;
import com.kxs.common.core.constant.enums.ErrorTypeEnum;
import com.kxs.common.core.constant.enums.UserStatusEnum;
import com.kxs.common.core.util.R;
import com.kxs.common.core.util.RetOps;
import com.kxs.common.security.model.KxsSecurityUser;
import com.kxs.llb.mech.api.feign.RemoteLlbUserService;
import com.kxs.llb.mech.api.model.LlbUser;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;

import java.util.Objects;

/**
 * 来量吧 商户端用户服务实现
 *
 * @author Lxq
 * @date 2023-12-06
 */
@Slf4j
@RequiredArgsConstructor
public class LlbMechUserServiceImpl implements KxsUserDetailsService{

    private final RemoteLlbUserService remoteLlbUserService;

    @Override
    @SneakyThrows
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {

        R<LlbUser> result = remoteLlbUserService.loadUserByPhone(username, SecurityConstants.FROM_IN);
        LlbUser custom = RetOps.of(result)
                .getData()
                .orElseThrow(() -> new UsernameNotFoundException(ErrorTypeEnum.USER_NOT_FOUND.getDescription()));

        if(Objects.equals(custom.getStatus(), UserStatusEnum.REMOVE.getCode())){
            throw new UsernameNotFoundException(ErrorTypeEnum.ACCOUNT_DISABLED.getDescription());
        }
        // 构造security用户
        return new KxsSecurityUser(custom.getId(), custom.getPhone(), SecurityConstants.BCRYPT + custom.getPassword(),  custom.getPhone(), custom.getSalt(),
               true, true, true, !Objects.equals(custom.getLockFlag(), UserStatusEnum.LOCKED.getCode()), AuthorityUtils.NO_AUTHORITIES);

    }

    /**
     * 是否支持此客户端校验
     * @param clientId 目标客户端
     * @param grantType  授权类型
     * @return true/false
     */
    @Override
    public boolean support(String clientId, String grantType) {
        return "merchant".equals(clientId);
    }

    @Override
    public int getOrder() {
        return Integer.MIN_VALUE + 1;
    }


}
